﻿#target photoshop
app.bringToFront();

var Layer = function() {}
var ArtLayer = function() {}
var Document = function() {}
var LayerSet = function() {}

File.prototype.partname = function() {
    var dotPosition = this.name.lastIndexOf(".");
    return this.name.substring(0, dotPosition);
}

Layer.prototype.layerWidth = function() {
    return (this.bounds[2].value - this.bounds[0].value);
}

Layer.prototype.layerHeight = function() {
    return (this.bounds[3].value - this.bounds[1].value);
}

ArtLayer.prototype.layerWidth = function() {
    return (this.bounds[2].value - this.bounds[0].value);
}

ArtLayer.prototype.layerHeight = function() {
    return (this.bounds[3].value - this.bounds[1].value);
}

Document.prototype.findLayerByName = function(layerName) {
    var result;
    try {
        result = this.layers.getByName(layerName);
    } catch (e) {}

    if (!result) {
        var sets = this.layerSets;
        for (var i = 0; i < sets.length; i++) {
            result = sets[i].findLayerByName(layerName);
            if (result) break;
        }
    }
    return result;
}

LayerSet.prototype.findLayerByName = function(layerName) {
    var result;
    try {
        result = this.layers.getByName(layerName);
    } catch (e) {}

    if (!result) {
        var sets = this.layerSets;
        for (var i = 0; i < sets.length; i++) {
            result = sets[i].getLayerByName(layerName);
            if (result) break;
        }
    }
    return result;
}

app.displayDialogs = DialogModes.NO;
myOldPrefs = app.preferences.rulerUnits;
app.preferences.rulerUnits = Units.PIXELS;

var itemLayerName = "Item";

var mySaveForWebOptions = new ExportOptionsSaveForWeb();
mySaveForWebOptions.quality = 75;
mySaveForWebOptions.interlaced = true;
mySaveForWebOptions.format = SaveDocumentType.JPEG;

var gSettings = new Object();
var savedSettings;
var okButtonID = 1;
var cancelButtonID = 2;

var scriptPath = File($.fileName).path;
var currentSettingsFile = File(scriptPath + "/crop_export.ini");

var myProgressBar;

// Error logging function
function logError(message) {
    var logFile = new File(scriptPath + "/error_log.txt");
    logFile.open("a");
    logFile.writeln(new Date().toUTCString() + ": " + message);
    logFile.close();
}

// Function to truncate the middle of long paths
function truncateMiddle(fullPath, maxLength) {
    if (fullPath.length <= maxLength) {
        return fullPath;
    }
    var partLength = Math.floor((maxLength - 3) / 2);
    return fullPath.substring(0, partLength) + '...' + fullPath.substring(fullPath.length - partLength);
}

// Function to show the main dialog and collect settings
function mainDialog() {
    // Methods
    this.validateSettings = function() {
        // Validate & populate globals
        var validated = false;
        do {
            var test;
            if (!gSettings.SourceFolder || !gSettings.SourceFolder.exists) {
                alert("Select source folder");
                break;
            }

            if (!gSettings.OutputFolder || !gSettings.OutputFolder.exists) {
                alert("Select output folder");
                break;
            }

            test = parseInt(jpgQuality.text);
            if (isNaN(test) || test < 0 || test > 100) {
                alert("Check JPG quality");
                break;
            }

            test = parseInt(productSize.text);
            if (isNaN(test)) {
                alert("Check Product size");
                break;
            }

            test = parseInt(thumbSize.text);
            if (isNaN(test)) {
                alert("Check Thumb size");
                break;
            }

            test = parseInt(paddingSize.text);
            if (isNaN(test)) {
                alert("Check Padding size");
                break;
            }

            test = parseInt(zoomSize.text);
            if (isNaN(test)) {
                alert("Check Zoom size");
                break;
            }

            validated = true;
        } while (false);

        if (validated) {
            gSettings.JpgQuality = parseInt(jpgQuality.text);
            gSettings.ZoomSize = parseInt(zoomSize.text);
            gSettings.ProductSize = parseInt(productSize.text);
            gSettings.ThumbSize = parseInt(thumbSize.text);
            gSettings.PaddingSize = parseInt(paddingSize.text);
        }
        return validated;
    }

    this.restoreSavedSettings = function(ss) {
        if (ss) {
            sourceFolderName.text = "";
            gSettings.SourceFolder = undefined;
            if (ss.SourceFolder) {
                sourceFolderName.text = truncateMiddle(ss.SourceFolder.fsName, 40);
                gSettings.SourceFolder = ss.SourceFolder;
                sourceFolderName.helpTip = ss.SourceFolder.fsName;
            }

            outputFolderName.text = "";
            gSettings.OutputFolder = undefined;
            if (ss.OutputFolder) {
                outputFolderName.text = truncateMiddle(ss.OutputFolder.fsName, 40);
                gSettings.OutputFolder = ss.OutputFolder;
                outputFolderName.helpTip = ss.OutputFolder.fsName;
            }

            if (ss.JpgQuality) {
                jpgQuality.text = ss.JpgQuality;
            }

            if (ss.ZoomSize)
                zoomSize.text = ss.ZoomSize;

            if (ss.ProductSize)
                productSize.text = ss.ProductSize;

            if (ss.ThumbSize)
                thumbSize.text = ss.ThumbSize;

            if (ss.PaddingSize)
                paddingSize.text = ss.PaddingSize;
        }
    }

    // End methods
    var dlg = new Window('dialog', 'PromoPilot: Crop and Export');

    var d = dlg;

    d.margins = 15;

    var gr1 = d.add('group');
    gr1.orientation = 'column';
    gr1.alignChildren = 'left';

    gr1.r1 = gr1.add('group');
    var sourceFolderButton = gr1.r1.add('button', undefined, 'Source Folder');
    var sourceFolderName = gr1.r1.add('edittext', undefined, "", {
        readonly: true
    });
    sourceFolderName.characters = 40;

    gr1.r2 = gr1.add('group');
    var outputFolderButton = gr1.r2.add('button', undefined, 'Output Folder');
    var outputFolderName = gr1.r2.add('edittext', undefined, "", {
        readonly: true
    });
    outputFolderName.characters = 40;

    gr1.r4 = gr1.add('group');
    gr1.r4.add('statictext', undefined, 'Product Image width:');
    var productSize = gr1.r4.add('edittext');
    productSize.text = "300";
    productSize.justify = "right";
    productSize.characters = 4;
    gr1.r4.add('statictext', undefined, 'px');
    gr1.r4.add('statictext', undefined, '    ');

    gr1.r4.add('statictext', undefined, 'Thumb width:');
    var thumbSize = gr1.r4.add('edittext');
    thumbSize.text = "100";
    thumbSize.justify = "right";
    thumbSize.characters = 4;
    gr1.r4.add('statictext', undefined, 'px');
    gr1.r4.add('statictext', undefined, '    ');

    gr1.r4.add('statictext', undefined, 'Zoom width:');
    var zoomSize = gr1.r4.add('edittext');
    zoomSize.text = "1000";
    zoomSize.justify = "right";
    zoomSize.characters = 4;
    gr1.r4.add('statictext', undefined, 'px');

    gr1.r5 = gr1.add('group');
    gr1.r5.add('statictext', undefined, 'Padding:');
    var paddingSize = gr1.r5.add('edittext');
    paddingSize.text = "5";
    paddingSize.justify = "right";
    paddingSize.characters = 2;
    gr1.r5.add('statictext', undefined, 'px');
    gr1.r5.add('statictext', undefined, '    ');

    gr1.r5.add('statictext', undefined, 'JPG Quality:');
    var jpgQuality = gr1.r5.add('edittext');
    jpgQuality.text = "75";
    jpgQuality.justify = "right";
    jpgQuality.characters = 3;

    d.c2 = d.add('group');
    d.c2.margins = 5;
    d.c2.orientation = 'row';
    d.c2.alignChildren = 'right';
    var okButton = d.c2.add('button', undefined, 'OK', {
        name: 'ok'
    });
    d.c2.add('button', undefined, 'Cancel', {
        name: 'cancel'
    });

    // Instructions, License, and Logo Section (added below the existing controls)
    var infoGroup = d.add('group');
    infoGroup.orientation = 'column';
    infoGroup.alignChildren = 'center';

    // Logo and Link
    var logo = infoGroup.add('image', undefined, File(scriptPath + "/PromoPilotLogo.png"));

    // Replace the button with static text
    infoGroup.add('statictext', undefined, "Visit bit.ly/pilot-photo-script for more information about this script.");

    // Events
    okButton.onClick = function() {
        if (validateSettings()) {
            d.close(okButtonID);  // Close the dialog before starting processing
        }
    }

    sourceFolderButton.onClick = function() {
        sourceFolder = Folder.selectDialog();
        if (sourceFolder) {
            gSettings.SourceFolder = sourceFolder;
            sourceFolderName.text = truncateMiddle(decodeURIComponent(sourceFolder.fsName), 60); // Truncate middle if needed
            sourceFolderName.helpTip = decodeURIComponent(sourceFolder.fsName); // Full path as tooltip
        }
    }

    outputFolderButton.onClick = function() {
        outputFolder = Folder.selectDialog();
        if (outputFolder) {
            gSettings.OutputFolder = outputFolder;
            outputFolderName.text = truncateMiddle(decodeURIComponent(outputFolder.fsName), 60); // Truncate middle if needed
            outputFolderName.helpTip = decodeURIComponent(outputFolder.fsName); // Full path as tooltip
        }
    }

    // Restore saved settings
    this.restoreSavedSettings(savedSettings);

    return d.show();
}

// Processing Workflow After OK Button
if (mainDialog() == okButtonID) {

    // Now the dialog has been closed, and processing can begin
    app.displayDialogs = DialogModes.NO;
    myOldPrefs = app.preferences.rulerUnits;
    app.preferences.rulerUnits = Units.PIXELS;

    // Close all opened files
    try {
        openDocs = app.documents;
        if (openDocs.length > 0) {
            var totalOpenedFiles = openDocs.length - 1;
            for (var i = totalOpenedFiles; i >= 0; i--) openDocs[i].close(SaveOptions.PROMPTTOSAVECHANGES);
        }
    } catch (e) {
        logError("Error closing documents: " + e.message);
    }

    // Saving settings
    try {
        currentSettingsFile.open("w");
        currentSettingsFile.write(gSettings.toSource());
        currentSettingsFile.close();
    } catch (e) {
        logError("Error saving settings: " + e.message);
    }

    // Setting jpg quality in save options
    mySaveForWebOptions.quality = gSettings.JpgQuality;

    // Process with progress bar
    try {
        totalFiles = totalFilesInFolder(gSettings.SourceFolder);
        var myProgressBarWindow = new Window('palette', 'Progress', [150, 150, 600, 300]);
        var myProgressBar = myProgressBarWindow.add('progressbar', [20, 35, 410, 60], 0, totalFiles);
        var progressCancelButton = myProgressBarWindow.add('button', [200, 80, 270, 100], 'Cancel');
        myProgressBarWindow.show();
        var executionCancelled = false;
        progressCancelButton.onClick = function() {
            alert("Execution cancelled by user");
            executionCancelled = true;
        }

        processAllFiles(gSettings.SourceFolder);
        myProgressBarWindow.close();
        alert("Done!");
    } catch (e) {
        logError("Error during processing: " + e.message);
        alert("An error occurred. Please check the error log.");
    }

    app.preferences.rulerUnits = myOldPrefs;
    app.displayDialogs = DialogModes.ALL;
}

// The rest of your script continues here...

function processAllFiles(myFolder) {
    var allFiles = myFolder.getFiles(isFile);

    if (allFiles !== null) {
        for (var i = 0; i < allFiles.length; i++) {

            if (!executionCancelled) {
                // Filter only image files
                var fileExt = allFiles[i].name.split('.').pop().toLowerCase();
                if (isValidImageFile(fileExt)) {
                    // Main processing
                    try {
                        // Open the document
                        var docRef = open(allFiles[i]);

                        // Handle PNGs with transparent backgrounds
                        if (fileExt === "png") {
                            trimTransparentPixels(docRef);  // Trim transparent pixels
                        }

                        // Resize and center image
                        resizeAndCenterImage(docRef, gSettings.ZoomSize, gSettings.PaddingSize);

                        // Save the resized image
                        saveResizedImages(docRef, allFiles[i].partname(), gSettings.OutputFolder);

                        docRef.close(SaveOptions.DONOTSAVECHANGES);
                    } catch (e) {
                        logError("Error processing file " + allFiles[i].name + ": " + e.message);
                    }

                    myProgressBar.value++;
                }
            } else {
                break;
            }
        }
    }
}

// Function to resize and center the image
function resizeAndCenterImage(docRef, targetSize, padding) {
    var newWidth, newHeight;

    // Determine scaling factor based on the larger dimension
    var currentSize = targetSize - 2 * padding;
    var widthRatio = currentSize / docRef.width.value;
    var heightRatio = currentSize / docRef.height.value;
    var scaleFactor = Math.min(widthRatio, heightRatio);

    // Apply the scaling factor
    newWidth = docRef.width.value * scaleFactor;
    newHeight = docRef.height.value * scaleFactor;
    docRef.resizeImage(UnitValue(newWidth, "px"), UnitValue(newHeight, "px"), undefined, ResampleMethod.BICUBICSHARPER);

    // Center the resized image within the canvas
    docRef.resizeCanvas(targetSize, targetSize, AnchorPosition.MIDDLECENTER);
}

// Function to save images at different sizes
function saveResizedImages(docRef, baseName, outputFolder) {
    var sizes = [gSettings.ZoomSize, gSettings.ProductSize, gSettings.ThumbSize];
    for (var i = 0; i < sizes.length; i++) {
        docRef.resizeImage(UnitValue(sizes[i], "px"), UnitValue(sizes[i], "px"), undefined, ResampleMethod.BICUBICSHARPER);
        var outputFile = new File(outputFolder.fullName + "/" + baseName + "-" + sizes[i] + ".jpg");
        docRef.exportDocument(outputFile, ExportType.SAVEFORWEB, mySaveForWebOptions);
    }
}

// Helper function to trim transparent pixels in PNGs
function trimTransparentPixels(docRef) {
    docRef.trim(TrimType.TRANSPARENT, true, true, true, true);
}

// Helper function to center non-transparent images
function centerLayer(docRef, itemLayer) {
    // Get center point
    var leftX = (itemLayer.bounds[0].value + itemLayer.bounds[2].value) / 2;
    var topY = (itemLayer.bounds[1].value + itemLayer.bounds[3].value) / 2;

    var rightX = docRef.width.value - leftX;
    var bottomY = docRef.height.value - topY;

    var dX = leftX - rightX;
    var dY = topY - bottomY;

    // X
    if (dX > 0)
        docRef.resizeCanvas(leftX * 2, docRef.height.value, AnchorPosition.MIDDLELEFT);

    if (dX < 0)
        docRef.resizeCanvas(rightX * 2, docRef.height.value, AnchorPosition.MIDDLERIGHT);

    // Y
    if (dY > 0)
        docRef.resizeCanvas(docRef.width.value, topY * 2, AnchorPosition.TOPCENTER);

    if (dY < 0)
        docRef.resizeCanvas(docRef.width.value, bottomY * 2, AnchorPosition.BOTTOMCENTER);
}

function totalFilesInFolder(myFolder) {
    var allFiles = myFolder.getFiles(isFile);
    var validFilesCount = 0;

    for (var i = 0; i < allFiles.length; i++) {
        var fileExt = allFiles[i].name.split('.').pop().toLowerCase();
        if (isValidImageFile(fileExt)) {
            validFilesCount++;
        }
    }
    return validFilesCount;
}

// Helper function to check if a file is a valid image file
function isValidImageFile(fileExt) {
    var validExtensions = ["jpg", "jpeg", "png", "gif", "bmp"];
    for (var i = 0; i < validExtensions.length; i++) {
        if (fileExt === validExtensions[i]) {
            return true;
        }
    }
    return false;
}

function isFolder(obj) {
    var result = false;
    if (obj.constructor.name === "Folder") result = true;
    return result;
}

function isFile(obj) {
    var result = false;
    if (obj.constructor.name === "File" && !obj.hidden) result = true;
    return result;
}

function pad(num, count) {
    var lenDiff = count - num.toString().length;
    var padding = "";

    if (lenDiff > 0)
        while (lenDiff--)
            padding += "0";

    return padding + num;
}
